'
' Programm: Morsekeyer mit Textspeicher
'
' Sprache: Bascom-AVR-Basic
'
' Erluterungen:
'
'
' Hardware:
'
' Tastkontakte fr Punkt und Strich an PB3 und PB2.(Tastung gegen Masse).
' Tonausgang (Buzzer) an PB0.
' Schaltausgang an PB1.
' Poti zur Geschwindigkeitseinstellung an PB4
' (Spannungsteiler aus 560 kOhm und 100kOhm log. - Schleifer an PB4).
' Starten des Textspeicher- Auslesens durch Reset- Taster
'
'
' Software:
'
' Nach Reset wird der Textspeicher ausgelesen; nach Textende 5 sec. Wartezeit, dann
' Neustart des Auslesens.
' Unterbrechung durch Bettigen des Punkt- Tastkontakts, danach geht die Schaltung in den
' Gebemodus.
' Im Gebemodus geht der Prozessor in Powerdown, wenn kein Kontakt mehr gedrckt ist
' Aufwecken durch PinChange Interrupt
'
' Timer0 erzeugt den Mithrton. Pin PB0 wird als Timer0-Ausgang OC0A verwendet.
' ADC liest Gleichspannung an PB4, sie wird in die Punktdauer umgerechnet
'
'
' Codierung der Morsezeichen des Textspeichers:
'
' 1 Byte pro Zeichen, niederwertiges Bit zuerst.
' Hat das Bit den Wert 0: Morsepunkt (dit)
' Hat das Bit den Wert 1: Moersestrich (dah)
' Da Morsezeichen unterschiedlich lang sind, ist das Ende- Zeichen ebenfalls ein 1-Bit.
'
' Beispiel Morsezeichen "a" (dit dah): &B00000110
' Beispiel Morsezeichen "9" (dah dah dah dah dit): &B00101111

' Auslesen der codierten Zeichen:
' niederwertigstes Bit auslesen, Byte um 1 Stelle nach rechts schieben, niederwertigstes Bit
' auslesen. Ist der Wert des gesamten Bytes nur noch 1, ist das Zeichen komplett.
'
'
'
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Grundeinstelllungen:
$regfile = "ATtiny13.dat"
$crystal = 1200000                                          ' Taktfrequenz: 1,2 MHz
$hwstack = 8                                                ' Platz fr nur 4 Unterprogramme reserviert, spart SRAM
$swstack = 0                                                ' wird nicht gebraucht
$framesize = 0                                              ' dito


'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Konstanten

Const Tonhoehe = 14                                         ' Tonhhe

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Variablen dimensionieren:

Dim I As Byte                                               ' wird in Zhlschleife verwendet
Dim N As Byte                                               ' Aktueller Morsebuchstabe
Dim M As Byte                                               ' wird frs Auslesen der Morsebits verwendet
Dim L As Byte                                               ' wird fr niederwertigstes Bit von M verwendet
Dim Dit As Word                                             ' Lnge Punkt
Dim Dah As Word                                             ' Lnge Strich
Dim Wort As Word                                            ' Lnge Wortabstand
Dim P As Bit                                                ' Punktspeicher


'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Initialisierung des Programms:
'

'On Int0 Ondit                                               ' Interruptroutine bei Drcken von Punkt- oder Strichkontakt

Acsr.7 = 1                                                  ' Analog-Komparator ausschalten, spart etwas Strom
Gimsk.5 = 1                                                 ' Pin Change Interrupt Enable
Sreg.7 = 1                                                  ' Global Interrupt Enable
Pcmsk.2 = 1                                                 ' Pin Change Inerrupt PB2 aktivieren
Pcmsk.3 = 1                                                 ' Pin Change Interrupt PB3 aktivieren
Ddrb = &B0000_0010                                          ' Pin PB1 als Ausgang konfigurieren (1), Rest ist Eingang (0)
Portb = &B0000_1100                                         ' PullUp-Widerstnde fr die Pins PB2 und PB3 aktivieren
'
'
' Morseton mit Timer0 erzeugen:

' Timer0 : Vorteilung(prescale) = 1 ; Bei Compare -match Toggle Pin Oc0a / Pb0
'
' Timer0 zhlt das Register TCNT0 hoch. Es wird mit Register OCR0A verglichen.
' Wenn TCNT0 = OCR0A, wird Output OC0A/PB0 umgeschaltet (getoggled)
' Timer0 erzeugt so Dauerton; er wird jedoch nur an den Ausgangspin durchgeschaltet,
' wenn das Datenrichtungs- Register DDRB.0 = 1 ist. Bei 0 wird der Ton unterdrckt.

Config Timer0 = Counter , Prescale = 64 , Compare A = Toggle , Clear Timer = 1
Ocr0a = Tonhoehe

' AD- Konverter konfigurieren und erst mal starten
Config Adc = Single , Prescaler = Auto
Start Adc

'-------------------------------------------------------------------------------
' Hauptprogramm:



Speicher:                                                   ' Erst mal Speicher auslesen

    Restore Message                                         ' Data- Zeiger auf Datenfeld 1 setzen
    For N = 1 To 128                                        ' Elemente des Datenfelds lesen (max. 128)
      Gosub Adclesen                                        ' Morsegeschwindigkeit abholen
      Read M                                                ' Zeichen aus Kette rauspicken
      If M = 255 Then                                       ' Ende- Zeichen
         Wait 5                                             ' Speichertext zu Ende,etwas warten
         Goto Speicher                                      ' Speichertext erneut auslesen
      End If
      If Pinb.3 = 0 Then                                    ' Abbruch, wenn Punkt- oder Strichhebel gedrckt
        Waitms 500
        Goto Keyer
      End If
      Gosub Morse                                           ' Morsezeichen erzeugen
    Next N


 Keyer:                                                     ' hier beginnt der hndische Tastenbetrieb

 Do

    Start Adc
    Gosub Adclesen
    If Pinb.3 = 0 Then                                      ' Punktkontakt gedrckt
      Ddrb.0 = 1                                            ' bei 1 wird Ausgang OC0A an Pin PB0 durchgeschaltet; Ton hrbar
      Portb.1 = 1                                           ' Schaltausgang ein
      Waitms Dit
      Ddrb.0 = 0                                            ' bei 0 wird Tonsignal unterdrckt
      Portb.1 = 0                                           ' Schaltausgang aus
      Waitms Dit
    End If

  Mark:

    If Pinb.2 = 0 Then                                      ' Strichkontakt gedrckt
      Ddrb.0 = 1                                            ' bei 1 wird Ausgang OC0A an Pin PB0 durchgeschaltet
      Portb.1 = 1
      Waitms Dah
      If Pinb.3 = 0 Then                                    ' Punktkontakt gedrckt
        P = 1                                               ' Squeeze- bzw. Punktspeicher
      End If
      Ddrb.0 = 0                                            ' bei 0 wird Tonsignal unterdrckt
      Portb.1 = 0
      Waitms Dit
      If P = 1 Then
        Ddrb.0 = 1                                          ' Squeeze  bzw. Punktspeicher auslesen
        Portb.1 = 1
        Waitms Dit
        Ddrb.0 = 0
        Portb.1 = 0                       ' war 1 
        Waitms Dit
        P = 0
        Goto Mark
      End If
    End If


Schlafen:

    If Pinb.2 = 1 And Pinb.3 = 1 Then                       ' Punkt- oder Strichkontakt sind nicht gedrckt
      Stop Adc                                              ' Strom sparen
      Powerdown                                             ' strkster Stromsparmodus (auch Taktoszillator wird gestoppt)
    End If

Loop



Morse:                                                      ' Morsezeichenerzeugung (nur Speicherbetrieb):


     If M = 0 Then                                          ' Sonderfall: Leerzeichen;
        Waitms Wort
     Goto Sign_end
     End If

     For I = 1 To 8
       If M = 1 Then                                        ' Das Byte hat nur noch den Wert 1; Zeichenende!
          Goto Sign_end
       End If
       L = M And &B0000001                                  ' niederwertigstes Bit lesen
       Ddrb.0 = 1                                           ' Ausgnge einschalten
       Portb.1 = 1
       If L = 1 Then
          Waitms Dah                                        ' ist das Bit 1 -> dah
         Else
          Waitms Dit                                        ' ist das Bit 0 -> dit
       End If
       Ddrb.0 = 0                                           ' Ausgnge abschalten
       Portb.1 = 0
       Waitms Dit                                           ' Pause innerhalb des Morsezeichens
       Shift M , Right , 1                                  ' Bits um eine Stelle nach rechts shiften
     Next I
     Sign_end:
     Waitms Dah                                             ' Pause zwischen Morsezeichen
   Return



Adclesen:

    Dit = Getadc(2)
    Dit = Dit + 32                                          ' fr sinnvollen Einstellbereich 100k-Poti + 560k gegen Plus
    'Dit = Dit + 256                                         ' stattdessen fr sinnvollen Einstellbereich 1M-Poti
    'Dit = Dit / 12                                          '             ""
    Dah = 3 * Dit
    Wort = 8 * Dit
  Return



End

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Codierung der Morsezeichen. Bei einem Prozessor mit viel SRAM wrde man alle Zeichen
' in ein Array einlesen (bersetzungstabelle), den eigentlichen Bakentext in einen String packen
' und diesen String zeichenenweise abarbeiten.
'
' Da der Attiny13 zu wenig SRAM hat und auch Flash knapp ist, ist der Weg hier anders:
' Die auszusendenden Zeichen sind direkt in der codierten Form in den Data- Zeilen ganz am Ende
' enthalten, allerdings in einer leicht lesbaren Schreibweise.
'
' Die bersetzungstabelle ist mit Konstanten realisiert. Nur die wirklich bentigten
' Zeichen werden (beim kompilieren des Programmes) umgewandelt.


Const #a = &B00000110
Const #b = &B00010001
Const #c = &B00010101
Const #d = &B00001001
Const #e = &B00000010
Const #f = &B00010100
Const #g = &B00001011
Const #h = &B00010000
Const #i = &B00000100
Const #j = &B00011110
Const #k = &B00001101
Const #l = &B00010010
Const #m = &B00000111
Const #n = &B00000101
Const #o = &B00001111
Const #p = &B00010110
Const #q = &B00011011
Const #r = &B00001010
Const #s = &B00001000
Const #t = &B00000011
Const #u = &B00001100
Const #v = &B00011000
Const #w = &B00001110
Const #x = &B00011001
Const #y = &B00011101
Const #z = &B00010011
Const #ae = &B00011010
Const #oe = &B00010111
Const #ue = &B00011100
Const #0 = &B00111111
Const #1 = &B00111110
Const #2 = &B00111100
Const #3 = &B00111000
Const #4 = &B00110000
Const #5 = &B00100000
Const #6 = &B00100001
Const #7 = &B00100011
Const #8 = &B00100111
Const #9 = &B00101111

Const Leerz = 0
Const Anfuehrz = &B01010010
Const Kl_auf = &B00101101
Const Kl_zu = &B01101101
Const Plus = &B00101010
Const Komma = &B01110011
Const Minus = &B01100001
Const Punkt = &B01101010
Const Slash = &B00101001
Const Dopp_pkt = &B01000111
Const Gleich = &B00110001
Const Fragez = &B01001100
Const Ende = 255




Message:

 Data #c , #q , Leerz , #c , #q , Leerz , #c , #q , Leerz , #d , #e , Leerz , #d , #k , #5 , #b , #u , Leerz , #d , #k , #5 , #b , #u , Leerz,
 Data #c , #q , Leerz , #c , #q , Leerz , #c , #q , Leerz , #d , #e , Leerz , #d , #k , #5 , #b , #u , Leerz , #d , #k , #5 , #b , #u , Leerz,
 Data #c , #q , Leerz , #c , #q , Leerz , #c , #q , Leerz , #d , #e , Leerz , #d , #k , #5 , #b , #u , Leerz , #d , #k , #5 , #b , #u , Leerz,
 Data Plus , #p , #s , #e , Leerz , #k ,
 Data Ende,